package com.ozs.service.entity;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.ozs.common.annotation.Excel;
import com.ozs.common.core.domain.BaseEntity;
import com.ozs.common.core.domain.entity.SysMenu;
import io.swagger.annotations.ApiModel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

/**
 * <p>
 * 相机台账管理表
 * </p>
 *
 * @author ozs
 * @since 2023-02-17
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
@ApiModel("相机台账管理表")
public class BaseCameraManagement extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键ID
     */
    @TableId(type = IdType.AUTO)
    private Long id;

    /**
     * 相机编码
     */
    @Excel(name = "相机编码")
    private String cameraCode;

    /**
     * 通道编号
     */
    @Excel(name = "通道编号")
    private String channel;

    /**
     * 所属工务段编码(单位id)
     */
    private Long deptId;

    /**
     * 行别(1上行2下行）
     */
    @Excel(name = "行别",readConverterExp = "1=上行,2=下行")
    private Integer lineDir;

    /**
     * 监控相机安装里程位置
     */
    private Integer installMile;

    /**
     * 监控范围开始里程位置
     */
    private Integer beginMile;

    /**
     * 监控范围结束里程位置
     */
    private Integer endMile;

    /**
     * 监视相机经度
     */
    @Excel(name = "监视相机经度")
    private String installLongitude;

    /**
     * 监视相机纬度
     */
    @Excel(name = "监视相机纬度")
    private String installLatitude;

    /**
     * 线路编码
     */
    private String railwayCode;

    /**
     * 物联网卡号
     */
    @Excel(name = "物联网卡号")
    private String cardNum;

    /**
     * 是否开启录像(1开启2不开启)
     */
    private Integer startRecording;

    /**
     * 录像接口返回的任务ID
     */
    private String taskId;


    /**
     * 监控相机安装里程位置字符串形式
     */
    @TableField(exist = false)
    @Excel(name = "监控相机安装里程位置")
    private String installMiles;

    /**
     * 监控范围开始里程位置字符串形式
     */
    @TableField(exist = false)
    @Excel(name = "监控范围开始里程位置")
    private String beginMiles;


    /**
     * 详情监控相机安装里程位置字符串形式
     */
    @TableField(exist = false)
    @Excel(name = "监控相机安装里程位置")
    private String installs;

    /**
     * 详情监控范围开始里程位置字符串形式
     */
    @TableField(exist = false)
    @Excel(name = "监控范围开始里程位置")
    private String begins;

    /**
     * 监控范围结束里程位置字符串形式
     */
    @TableField(exist = false)
    @Excel(name = "监控范围结束里程位置")
    private String endMiles;

    /**
     *线路名称
     */
    @TableField(exist = false)
    @Excel(name = "线路名称")
    private String railwayName;

    /**
     *所属工务段名称
     */
    @TableField(exist = false)
    @Excel(name = "所属工务段名称")
    private String deptName;

    /**
     *是否显示启用录像按钮(1显示2不显示)
     */
    @TableField(exist = false)
    private Integer isStartRecording;
}