package com.ozs.web.controller.accountmanagment;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.annotation.Log;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.core.domain.entity.SysUser;
import com.ozs.common.core.domain.model.LoginUser;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.utils.AppendUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.file.FileUtils;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.service.entity.*;
import com.ozs.service.entity.vo.*;
import com.ozs.service.mapper.MsgAlarmMapper;
import com.ozs.service.service.*;
import com.ozs.system.service.DataScoreUtil;
import com.ozs.system.service.ISysDeptService;
import com.ozs.system.service.ISysDictDataService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 报警信息表 前端控制器
 * </p>
 *
 * @author ozs
 * @since 2023-02-22
 */
@RestController
@RequestMapping("/service/msgAlarm")
public class MsgAlarmController extends BaseController {
    private static final Logger log = LoggerFactory.getLogger(MsgAlarmController.class);
    @Resource
    MsgAlarmService msgAlarmService;
    @Resource
    MsgAlarmMapper msgAlarmMapper;
    @Resource
    MsgAlarmFrequencyService msgAlarmFrequencyService;
    @Autowired
    BaseCameraManagementService baseCameraManagementService;
    @Autowired
    BaseRailwayManagementService baseRailwayManagementService;
    @Autowired
    private MsgWebPushService msgWebPushService;
    @Autowired
    private ISysDictDataService dictDataService;
    @Autowired
    private DataScoreUtil dataScoreUtil;
    @Resource
    private BaseUserService baseUserService;
    @Autowired
    private ISysDeptService deptService;

    /**
     * web更多消息-总数，已读数
     */
    @PostMapping("/messageMoreNum")
    @ApiOperation("更多消息--总数，已读数")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult messageMoreNum(@RequestBody MsgAlarmVo msgAlarmVo) {
        // MsgAlarmVo msgAlarmVo1 = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        Map<String, Integer> messageMore = msgWebPushService.messageMoreNum(getUserId());
        return AjaxResult.success(messageMore);
    }

    /**
     * web更多消息
     */
    @PostMapping("/messageMore")
    @ApiOperation("更多消息")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult messageMore(@RequestBody MsgAlarmVo msgAlarmVo) {
        // MsgAlarmVo msgAlarmVo1 = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        String userId = getUserId();
        if (!StringUtils.isEmpty(userId)) {
            msgAlarmVo.setUserId(userId);
        }
        IPage<MsgAlarm> messageMore = msgWebPushService.messageMore(msgAlarmVo);
        return AjaxResult.success(messageMore);
    }

    /**
     * web报警未读消息个数+未读消息列表
     */
    @GetMapping("/webAlarm")
    @ApiOperation("app报警推送次数")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult webAlarmNum() {
        // MsgAlarmVo msgAlarm = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), new MsgAlarmVo());
        Map webAlarm = msgWebPushService.webAlarm(getUserId());
        return AjaxResult.success(webAlarm);
    }

    /**
     * web消息全部已读
     */
    @PostMapping("/readAll")
    @ApiOperation(value = "web消息全部已读")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult readAll() {
        return toAjax(msgWebPushService.readAll(getUserId()));
    }

    /**
     * web消息已读
     */
    @PostMapping("/read")
    @ApiOperation(value = "web消息已读")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult read(@RequestBody MsgWebPush msgWebPush) {
//        if (StringUtils.isEmpty(msgWebPush) || ObjectUtils.isEmpty(msgWebPush.getPushId())) {
//            return AjaxResult.error("PushId不能为空！");
//        }
        return toAjax(msgWebPushService.read(msgWebPush));
    }

    /**
     * 报警信息分页
     *
     * @param msgAlarmVo
     * @return
     */
    @ApiOperation(value = "报警信息分页")
    @PostMapping("/list")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult selectMsgAlarm(@RequestBody MsgAlarmVo msgAlarmVo) {
        msgAlarmVo = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        if (!ObjectUtils.isEmpty(msgAlarmVo.getAlarmMile())) {
                msgAlarmVo.setAlarmMile(msgAlarmVo.getAlarmMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getBeginMile())) {
                msgAlarmVo.setBeginMile(msgAlarmVo.getBeginMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getEndMile())) {
                msgAlarmVo.setEndMile(msgAlarmVo.getEndMile() * 1000);
        }
        IPage<MsgAlarm> page = msgAlarmService.queryPage(msgAlarmVo);
        page.setTotal(page.getTotal());
        page.setCurrent(page.getCurrent());
        page.setPages(page.getPages());

        if (!ObjectUtils.isEmpty(page) && page.getRecords().size() > 0) {
            List<MsgAlarm> dto1 = page.getRecords().stream().map(o -> {
                String[] split = o.getImageUrl().split(";");
                ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
                o.setImageUrls(objects);
                LambdaQueryWrapper<BaseCameraManagement> cameraManagementLambdaQueryWrapper = new LambdaQueryWrapper<BaseCameraManagement>();
                if (!ObjectUtils.isEmpty(o.getCameraCode())) {
                    cameraManagementLambdaQueryWrapper.eq(BaseCameraManagement::getCameraCode, o.getCameraCode());
                }
                BaseCameraManagement cameraManagement = baseCameraManagementService.getOne(cameraManagementLambdaQueryWrapper);
                LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
                if (!ObjectUtils.isEmpty(cameraManagement.getRailwayCode())) {
                    queryWrapper.eq(BaseRailwayManagement::getRailwayCode, cameraManagement.getRailwayCode());
                }
                BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
                o.setRailwayName(baseRailwayManagement.getRailwayName());
                if (StringUtils.isNotEmpty(o.getReleasedBy())) {
                    BaseUser user = baseUserService.getUser(o.getReleasedBy());
                    o.setReleasedByName(user.getNickName());
                }
                String mils = AppendUtils.stringAppend(cameraManagement.getInstallMile());
                o.setAlarmMiles(mils);
                String end = AppendUtils.stringAppend(cameraManagement.getEndMile());
                o.setEndMile(end);
                String begin = AppendUtils.stringAppend(cameraManagement.getBeginMile());
                o.setBeginMile(begin);
                LambdaQueryWrapper<MsgAlarmFrequency> wrapper = new LambdaQueryWrapper<MsgAlarmFrequency>();
                if (!ObjectUtils.isEmpty(o.getAlarmId())) {
                    wrapper.eq(MsgAlarmFrequency::getAlarmId, o.getAlarmId());
                }
                o.setAlarmCount((int) msgAlarmFrequencyService.count(wrapper));
                if (!ObjectUtils.isEmpty(o.getReleasedType())) {
                    o.setReleasedTypeName(o.getReleasedType() == 1 ? "真实报警" : "误报警");
                }
                return o;
            }).collect(Collectors.toList());
            page.setRecords(dto1);
        }
        return AjaxResult.success(page);
    }
    
    /**
     * 导出报警消息
     *
     * @param response
     */
    @ApiOperation("导出报警消息")
    @PostMapping("/exportMsgAlarm")
    @Log(title = "报警信息管理", businessType = BusinessType.EXPORT)
    public void exportMsgAlarm(HttpServletResponse response, @RequestBody MsgAlarmVo msgAlarmVo) {
        msgAlarmVo = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        if (!ObjectUtils.isEmpty(msgAlarmVo.getAlarmMile())) {
            msgAlarmVo.setAlarmMile(msgAlarmVo.getAlarmMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getBeginMile())) {
            msgAlarmVo.setBeginMile(msgAlarmVo.getBeginMile() * 1000);
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getEndMile())) {
            msgAlarmVo.setEndMile(msgAlarmVo.getEndMile() * 1000);
        }
        List<MsgAlarm> msgAlarmList = msgAlarmService.msgAlarmList(msgAlarmVo);
        for (MsgAlarm msgAlarm : msgAlarmList) {
            String[] split = msgAlarm.getImageUrl().split(";");
            ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
            msgAlarm.setImageUrls(objects);
            LambdaQueryWrapper<BaseCameraManagement> cameraManagementLambdaQueryWrapper = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
                cameraManagementLambdaQueryWrapper.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
            }
            BaseCameraManagement cameraManagement = baseCameraManagementService.getOne(cameraManagementLambdaQueryWrapper);
            LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(cameraManagement.getRailwayCode())) {
                queryWrapper.eq(BaseRailwayManagement::getRailwayCode, cameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
            msgAlarm.setRailwayName(baseRailwayManagement.getRailwayName());
            if (StringUtils.isNotEmpty(msgAlarm.getReleasedBy())) {
                BaseUser user = baseUserService.getUser(msgAlarm.getReleasedBy());
                msgAlarm.setReleasedByName(user.getNickName());
            }
            String mils = AppendUtils.stringAppend(cameraManagement.getInstallMile());
            msgAlarm.setAlarmMiles(mils);
            String end = AppendUtils.stringAppend(cameraManagement.getEndMile());
            msgAlarm.setEndMile(end);
            String begin = AppendUtils.stringAppend(cameraManagement.getBeginMile());
            msgAlarm.setBeginMile(begin);
            LambdaQueryWrapper<MsgAlarmFrequency> wrapper = new LambdaQueryWrapper<MsgAlarmFrequency>();
            if (!ObjectUtils.isEmpty(msgAlarm.getAlarmId())) {
                wrapper.eq(MsgAlarmFrequency::getAlarmId, msgAlarm.getAlarmId());
            }
            msgAlarm.setAlarmCount((int) msgAlarmFrequencyService.count(wrapper));
            if (!ObjectUtils.isEmpty(msgAlarm.getReleasedType())) {
                msgAlarm.setReleasedTypeName(msgAlarm.getReleasedType() == 1 ? "真实报警" : "误报警");
            }
            msgAlarm.setLineDirName(msgAlarm.getLineDir() == 1 ? "上行" : "下行");
            msgAlarm.setIsLockName(msgAlarm.getIsLock() == 1 ? "已解除" : "未解除");
            msgAlarm.setAlarmTypeName("泥石流");
        }
        ExcelUtil<MsgAlarm> util = new ExcelUtil<>(MsgAlarm.class);
        util.exportExcel(response, msgAlarmList, "报警信息管理");
    }

    @GetMapping(value = "/getByAlarmId/{alarmId}")
    @ApiOperation("根据报警信息ID获取详细信息-app")
    @Log(title = "报警信息管理", businessType = BusinessType.MESSAGE)
    public AjaxResult getByAlarmId(@PathVariable String alarmId) {
        MsgAlarm msgAlarm = msgAlarmService.getByAlarmId(alarmId);
        if (!ObjectUtils.isEmpty(msgAlarm)) {
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
                lw.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
            }
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
            LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
            MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
            String[] split = msgAlarm.getImageUrl().split(";");
            ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
            msgAlarm.setImageUrls(objects);
            if (StringUtils.isNotEmpty(msgAlarm.getReleasedBy())) {
                BaseUser user = baseUserService.getUser(msgAlarm.getReleasedBy());
                msgAlarm.setReleasedByName(user.getNickName());
            }
            BeanUtils.copyProperties(msgAlarm, msgAlarmResp);
            msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
            msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
            String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
            String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
            String endMile = begin + "-" + end;
            msgAlarmResp.setBeginMile(endMile);
            String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            msgAlarmResp.setAlarmMile(mile);
            msgAlarmResp.setInstallLongitude(baseCameraManagement.getInstallLongitude() + "," + baseCameraManagement.getInstallLatitude());
            return AjaxResult.success(msgAlarmResp);
        } else {
            return success("无报警信息数据");
        }
    }

    @GetMapping(value = "/{id}")
    @ApiOperation("根据报警信息ID获取详细信息")
    @Log(title = "报警信息管理", businessType = BusinessType.MESSAGE)
    public AjaxResult getInfo(@PathVariable Long id) {
        MsgAlarm msgAlarm = msgAlarmService.getById(id);
        if (!ObjectUtils.isEmpty(msgAlarm)) {
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
                lw.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
            }
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
            LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
            MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
            String[] split = msgAlarm.getImageUrl().split(";");
            ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
            msgAlarm.setImageUrls(objects);
            if (StringUtils.isNotEmpty(msgAlarm.getReleasedBy())) {
                BaseUser user = baseUserService.getUser(msgAlarm.getReleasedBy());
                msgAlarm.setReleasedByName(user.getNickName());
            }
            BeanUtils.copyProperties(msgAlarm, msgAlarmResp);
            msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
            msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
            String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
            String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
            String endMile = "[" + begin + "]-[" + end + "]";
            msgAlarmResp.setBeginMile(endMile);
            String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            msgAlarmResp.setAlarmMile(mile);
            msgAlarmResp.setInstallLongitude(baseCameraManagement.getInstallLongitude() + "," + baseCameraManagement.getInstallLatitude());
            SysDept sysDept = deptService.selectDeptById(baseCameraManagement.getDeptId());
            msgAlarmResp.setDeptName(sysDept.getDeptName());
            return AjaxResult.success(msgAlarmResp);
        } else {
            return success("无报警信息数据");
        }
    }

    @PutMapping(value = "/updateIsLock")
    @ApiOperation("根据报警信息ID解除报警")
    @Log(title = "报警信息管理", businessType = BusinessType.UPDATE)
    public AjaxResult updateIsLock(@RequestParam(value = "id") Long id,
                                   @RequestParam(value = "releasedType",required = false) Integer releasedType,
                                   @RequestParam(value = "releasedReason",required = false) String releasedReason) {
        MsgAlarm msgAlarm = msgAlarmService.getById(id);
        msgAlarm.setIsLock(1);
        msgAlarm.setUpdateBy(getUserId());
        msgAlarm.setReleasedBy(getUserId());
        msgAlarm.setReleasedTime(new Date());
        if (!ObjectUtils.isEmpty(releasedReason)) {
            msgAlarm.setReleasedReason(releasedReason);
        }
        if (!ObjectUtils.isEmpty(releasedType)) {
            msgAlarm.setReleasedType(releasedType);
        }
        return AjaxResult.success(msgAlarmService.updateById(msgAlarm));
    }

    @GetMapping(value = "/alarmHistoricalAlarm/{alarmId}")
    @ApiOperation("报警消息详情历史报警次数信息")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult alarmHistoricalAlarm(@PathVariable Long alarmId) {
        QueryWrapper<MsgAlarmFrequency> wrapper = new QueryWrapper<>();
        wrapper.eq("alarm_id", alarmId);
        wrapper.orderByDesc("alarm_time");
        List<MsgAlarmFrequency> list = msgAlarmFrequencyService.list(wrapper);
        return AjaxResult.success(list);
    }

    @GetMapping(value = "/videoHistoricalAlarm/{cameraCode}")
    @ApiOperation("视频服务历史报警信息")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult videoHistoricalAlarm(@PathVariable String cameraCode) {
        QueryWrapper<MsgAlarm> wrapper = new QueryWrapper<>();
        wrapper.eq("camera_code", cameraCode);
        wrapper.orderByDesc("alarm_time");
        List<MsgAlarm> list = msgAlarmService.list(wrapper);
        return AjaxResult.success(list);
    }

    /**
     * web数据统计
     *
     * @param alarmStatisticResVo
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.OTHER)
    @PostMapping("/dataStatistic")
    @ApiOperation(value = "web数据统计 报警数据详情")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "railwayCode", value = "线路编码"),
            @ApiImplicitParam(paramType = "query", name = "currentYear", value = "当前年份"),
            @ApiImplicitParam(paramType = "query", name = "currentMonth", value = "当前月份"),
            @ApiImplicitParam(paramType = "query", name = "alarmType", value = "灾害类型"),
    })
    public AjaxResult dataStatistic(@RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        Integer pageNum = alarmStatisticResVo.getPageNum().intValue();
        Integer pageSize = alarmStatisticResVo.getPageSize().intValue();
        SysUser user = getLoginUser().getUser();
        String userId = getUserId();
        if (user.isAdmin(user.getId())) {
            userId = "";
        }
        List<AlarmStatisticVo> list = msgAlarmService.list(alarmStatisticResVo.getRailwayCode(), alarmStatisticResVo.getCurrentYear(),
                alarmStatisticResVo.getCurrentMonth(), alarmStatisticResVo.getAlarmType(), pageNum, pageSize, userId);
        if (!ObjectUtils.isEmpty(list)) {
            list.forEach(l -> {
                if (!ObjectUtils.isEmpty(l.getAlarmMile())) {
                    l.setAlarmMile(AppendUtils.stringAppend(Integer.valueOf(l.getAlarmMile())));
                }
                if (StringUtils.isNotEmpty(l.getReleasedBy())) {
                    BaseUser buser = baseUserService.getUser(l.getReleasedBy());
                    l.setReleasedByName(buser.getNickName());
                }
                if (!ObjectUtils.isEmpty(l.getReleasedType())) {
                    l.setReleasedTypeName(l.getReleasedType() == 1 ? "真实报警" : "误报警");
                }
                if (!ObjectUtils.isEmpty(l.getLineDir())) {
                    l.setLineDirName(l.getLineDir() == 1 ? "上行" : "下行");
                }
                if (!ObjectUtils.isEmpty(l.getAlarmType())) {
                    l.setAlarmTypeName(dictDataService.selectDictLabel("sys_alarm_type", String.valueOf(l.getAlarmType())));
                }
                if (!ObjectUtils.isEmpty(l.getIsLock())) {
                    l.setIsLockName(l.getIsLock() == 1 ? "已解除" : "未解除");
                }
            });
        }
        IPage page = msgAlarmService.listToPage(list, pageNum, pageSize);
        return AjaxResult.success(page);
    }

    /**
     * web数据统计
     *
     * @param
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.OTHER)
    @PostMapping("/alarmMonthStatistic")
    @ApiOperation(value = "web数据统计 报警月统计数据")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "railwayCode", value = "线路编码"),
            @ApiImplicitParam(paramType = "query", name = "currentYear", value = "当前年份"),
            @ApiImplicitParam(paramType = "query", name = "currentMonth", value = "当前月份"),
            @ApiImplicitParam(paramType = "query", name = "alarmType", value = "灾害类型"),
    })
    public AjaxResult alarmMonthStatistic(@RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        SysUser user = getLoginUser().getUser();
        String userId = getUserId();
        ;
        if (user.isAdmin(user.getId())) {
            userId = "";
        }
        List<Integer> numbs = new ArrayList<>();
        Integer listSize;
        List<AlarmStatisticMonthDto> list = null;
        if (ObjectUtils.isEmpty(alarmStatisticResVo.getCurrentMonth())) {
            list = msgAlarmService.getCount(alarmStatisticResVo.getRailwayCode(), alarmStatisticResVo.getCurrentYear(),
                    alarmStatisticResVo.getCurrentMonth(), alarmStatisticResVo.getAlarmType(), userId);
            listSize = 12;
        } else {
            if (alarmStatisticResVo.getCurrentMonth().equals(2)) {//2
                listSize = 28;
            } else if (alarmStatisticResVo.getCurrentMonth().equals(4) || alarmStatisticResVo.getCurrentMonth().equals(6) ||
                    alarmStatisticResVo.getCurrentMonth().equals(9) || alarmStatisticResVo.getCurrentMonth().equals(11)) {//4,6,9,11
                listSize = 30;
            } else {//1,3,5,7,8,10,12
                listSize = 31;
            }
            list = msgAlarmService.getDayCount(alarmStatisticResVo.getRailwayCode(), alarmStatisticResVo.getCurrentYear(),
                    alarmStatisticResVo.getCurrentMonth(), alarmStatisticResVo.getAlarmType(), userId);
        }
        for (int i = 1; i <= listSize; i++) {
            numbs.add(i);
        }

        List<Integer> list1 = new ArrayList<>();
            if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
                list.forEach(l -> {
                    list1.add(l.getMonthsta());
                });
            }
            AlarmStatisticMonthDto dto;
            for (int i = 0; i < numbs.size(); i++) {
                if (!list1.contains(numbs.get(i))) {
                    dto = new AlarmStatisticMonthDto();
                    dto.setMonthsta(numbs.get(i));
                    dto.setFrequency(0);
                    list.add(dto);
                }
            }
            if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
                Collections.sort(list, new Comparator<AlarmStatisticMonthDto>() {
                    @Override
                    public int compare(AlarmStatisticMonthDto o1, AlarmStatisticMonthDto o2) {

                        return o1.getMonthsta().compareTo(o2.getMonthsta());
                    }
                });
            }
            return AjaxResult.success(list);
    }


    /**
     * web数据统计 数据导出
     *
     * @param
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.EXPORT)
    @PostMapping("/exportDataStatistic")
    @ApiOperation(value = "web数据统计-数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "railwayCode", value = "线路编码"),
            @ApiImplicitParam(paramType = "query", name = "currentYear", value = "当前年份"),
            @ApiImplicitParam(paramType = "query", name = "currentMonth", value = "当前月份"),
            @ApiImplicitParam(paramType = "query", name = "alarmType", value = "灾害类型"),
    })
    public void exportDataStatistic(HttpServletResponse response, @RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        SysUser user = getLoginUser().getUser();
        String userId = getUserId();
        if (user.isAdmin(user.getId())) {
            userId = "";
        }
        try {
            List<AlarmStatisticDto> list = msgAlarmService.exportExcel(alarmStatisticResVo, userId);
            if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
                list.forEach(l -> {
                    if (!ObjectUtils.isEmpty(l.getAlarmMile())) {
                        l.setAlarmMiles(AppendUtils.stringAppend(Integer.valueOf(l.getAlarmMile())));
                    }
                    if (StringUtils.isNotEmpty(l.getReleasedBy())) {
                        BaseUser buser = baseUserService.getUser(l.getReleasedBy());
                        l.setReleasedByName(buser.getNickName());
                    }
                    if (!ObjectUtils.isEmpty(l.getReleasedType())) {
                        l.setReleasedTypeName(l.getReleasedType() == 1 ? "真实报警" : "误报警");
                    }
                    if (!ObjectUtils.isEmpty(l.getAlarmType())) {
                        String alarmType = l.getAlarmType();
                        String alarmTypeValue = dictDataService.selectDictLabel("sys_alarm_type", alarmType);
                        l.setAlarmTypeValue(alarmTypeValue);
                    }
                    if (!ObjectUtils.isEmpty(l.getLineDir())) {
                        String lineDir = l.getLineDir();
                        String lineDirValue = dictDataService.selectDictLabel("sys_line_dir", lineDir);
                        l.setLineDirValue(lineDirValue);
                    }
                    if (!ObjectUtils.isEmpty(l.getIsLock())) {
                        l.setIsLockName(l.getIsLock() == 1 ? "已解除" : "未解除");
                    }
                });
            }

            ExcelUtil<AlarmStatisticDto> util = new ExcelUtil<>(AlarmStatisticDto.class);
            util.exportExcel(response, list, "报警数据详情");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
        }
    }

    /**
     * web数据统计 报警月统计数据导出
     *
     * @param
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.EXPORT)
    @PostMapping("/exportDataStatisticMonth")
    @ApiOperation(value = "报警月统计数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "railwayCode", value = "线路编码"),
            @ApiImplicitParam(paramType = "query", name = "currentYear", value = "当前年份"),
            @ApiImplicitParam(paramType = "query", name = "currentMonth", value = "当前月份"),
            @ApiImplicitParam(paramType = "query", name = "alarmType", value = "灾害类型"),
    })
    public void exportDataStatisticMonth(HttpServletResponse response, @RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        SysUser user = getLoginUser().getUser();
        String userId = getUserId();
        if (user.isAdmin(user.getId())) {
            userId = "";
        }

        List<AlarmStatisticMonthDto> list = null;
        if (ObjectUtils.isEmpty(alarmStatisticResVo.getCurrentMonth())) {
            list = msgAlarmService.getCount(alarmStatisticResVo.getRailwayCode(), alarmStatisticResVo.getCurrentYear(),
                    alarmStatisticResVo.getCurrentMonth(), alarmStatisticResVo.getAlarmType(), userId);
        } else {
            list = msgAlarmService.getDayCount(alarmStatisticResVo.getRailwayCode(), alarmStatisticResVo.getCurrentYear(),
                    alarmStatisticResVo.getCurrentMonth(), alarmStatisticResVo.getAlarmType(), userId);
        }
        ExcelUtil<AlarmStatisticMonthDto> util = new ExcelUtil<>(AlarmStatisticMonthDto.class);
        util.exportExcel(response, list, "报警月统计数据");
    }
}

