package com.ozs.web.controller.system;

import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.core.domain.vo.SysDictDataVo;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ozs.common.annotation.Log;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDictData;
import com.ozs.common.core.page.TableDataInfo;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.system.service.ISysDictDataService;
import com.ozs.system.service.ISysDictTypeService;

/**
 * 数据字典信息
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/dict/data")
public class SysDictDataController extends BaseController {
    @Autowired
    private ISysDictDataService dictDataService;

    @Autowired
    private ISysDictTypeService dictTypeService;

//    @PreAuthorize("@ss.hasPermi('system:dict:list')")
//    @GetMapping("/list")
//    public TableDataInfo list(SysDictData dictData)
//    {
//        startPage();
//        List<SysDictData> list = dictDataService.selectDictDataList(dictData);
//        return getDataTable(list);
//    }

    /**
     * 获取字典数据列表
     *
     * @param sysDictDataVo
     * @return
     */
    @ApiOperation(value = "获取字典类型列表")
    @PostMapping("/list")
    @PreAuthorize("@ss.hasPermi('system:dict:list')")
    public AjaxResult list(@RequestBody SysDictDataVo sysDictDataVo) {
        LambdaQueryWrapper<SysDictData> wrapper = new LambdaQueryWrapper<SysDictData>();
        if (!ObjectUtils.isEmpty(sysDictDataVo.getDictCode())) {
            wrapper.like(SysDictData::getDictCode, sysDictDataVo.getDictCode());
        }
        if (!ObjectUtils.isEmpty(sysDictDataVo.getDictSort())) {
            wrapper.like(SysDictData::getDictSort, sysDictDataVo.getDictSort());
        }
        if (!ObjectUtils.isEmpty(sysDictDataVo.getDictLabel())) {
            wrapper.like(SysDictData::getDictLabel, sysDictDataVo.getDictLabel());
        }
        if (!ObjectUtils.isEmpty(sysDictDataVo.getDictValue())) {
            wrapper.like(SysDictData::getDictValue, sysDictDataVo.getDictValue());
        }
        if (!ObjectUtils.isEmpty(sysDictDataVo.getDictType())) {
            wrapper.like(SysDictData::getDictType, sysDictDataVo.getDictType());
        }
        if (!ObjectUtils.isEmpty(sysDictDataVo.getCssClass())) {
            wrapper.like(SysDictData::getCssClass, sysDictDataVo.getCssClass());
        }
        if (!ObjectUtils.isEmpty(sysDictDataVo.getListClass())) {
            wrapper.like(SysDictData::getListClass, sysDictDataVo.getListClass());
        }
        if (!ObjectUtils.isEmpty(sysDictDataVo.getIsDefault())) {
            wrapper.like(SysDictData::getIsDefault, sysDictDataVo.getIsDefault());
        }
        if (!ObjectUtils.isEmpty(sysDictDataVo.getStatus())) {
            wrapper.like(SysDictData::getStatus, sysDictDataVo.getStatus());
        }
        IPage<SysDictData> page = dictDataService.page(new Page<>(sysDictDataVo.getPageNum(), sysDictDataVo.getPageSize()), wrapper);
        return AjaxResult.success(page);
    }

    @Log(title = "字典数据", businessType = BusinessType.EXPORT)
    @PreAuthorize("@ss.hasPermi('system:dict:export')")
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SysDictData dictData) {
        List<SysDictData> list = dictDataService.selectDictDataList(dictData);
        ExcelUtil<SysDictData> util = new ExcelUtil<SysDictData>(SysDictData.class);
        util.exportExcel(response, list, "字典数据");
    }

    /**
     * 查询字典数据详细
     */
    @PreAuthorize("@ss.hasPermi('system:dict:query')")
    @GetMapping(value = "/{dictCode}")
    public AjaxResult getInfo(@PathVariable Long dictCode)
    {
        return success(dictDataService.selectDictDataById(dictCode));
    }

    /**
     * 根据字典类型查询字典数据信息
     */
        @GetMapping(value = "/type/{dictType}")
    public AjaxResult dictType(@PathVariable String dictType)
    {
        List<SysDictData> data = dictTypeService.selectDictDataByType(dictType);
        if (StringUtils.isNull(data))
        {
            data = new ArrayList<SysDictData>();
        }
        return success(data);
    }

    /**
     * 新增字典类型
     */
    @PreAuthorize("@ss.hasPermi('system:dict:add')")
    @Log(title = "字典数据", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysDictData dict)
    {
        dict.setCreateBy(getUsername());
        return toAjax(dictDataService.insertDictData(dict));
    }

    /**
     * 修改保存字典类型
     */
    @PreAuthorize("@ss.hasPermi('system:dict:edit')")
    @Log(title = "字典数据", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SysDictData dict)
    {
        dict.setUpdateBy(getUsername());
        return toAjax(dictDataService.updateDictData(dict));
    }

    /**
     * 删除字典类型
     */
    @PreAuthorize("@ss.hasPermi('system:dict:remove')")
    @Log(title = "字典类型", businessType = BusinessType.DELETE)
    @DeleteMapping("/{dictCodes}")
    public AjaxResult remove(@PathVariable Long[] dictCodes)
    {
        dictDataService.deleteDictDataByIds(dictCodes);
        return success();
    }
}
