package com.ozs.web.core.util;


import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ozs.common.config.BaseConfig;
import com.ozs.common.constant.Constants;
import com.ozs.common.core.domain.entity.SysDictData;
import com.ozs.common.exception.base.BaseException;
import com.ozs.common.utils.DateUtils;
import com.ozs.common.utils.http.HttpUtils;
import com.ozs.service.entity.BaseCameraManagement;
import com.ozs.service.service.BaseCameraManagementService;
import com.ozs.system.mapper.SysDictDataMapper;
import com.ozs.web.core.config.CaneraConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * 相机工具
 */

@Configuration
@Slf4j
@Component
public class CameraUtil {

    private static String historyUrl;
    private static String ffmpegPath;
    private static String filePath;
    private static String transcribeFilePath;
    private static String webUrl;
    private static String bakUrl;
    private static CmdCameraUtil cUtil;

    @Autowired
    private CaneraConfig caneraConfig;

    @Autowired
    private CmdCameraUtil cmdCameraUtil;

    @Autowired
    private SysDictDataMapper dictDataMapper;
    @Resource
    BaseCameraManagementService baseCameraManagementService;


    /**
     * 历史回放
     *
     * @param fromVideoFileList
     * @param ph
     * @return
     * @throws IOException
     */
    public static String historyPlay(List<String> fromVideoFileList, String ph, String uuid) {
        // 视频服务映射路径
        String NewfilePath = BaseConfig.getProfile() + "/" + ph;
        log.info("NewfilePath:{}", NewfilePath);
        log.info("fromVideoFileList:{}", fromVideoFileList);
        if (ObjectUtils.isEmpty(fromVideoFileList) || fromVideoFileList.size() <= 0) {
            throw new BaseException("当前相机无视频录像");
        }
        ExecutorService executor = Executors.newFixedThreadPool(10);

        executor.submit(new Runnable() {
            @Override
            public void run() {
                try {
                    Map<String, String> map = myConvetor(fromVideoFileList, NewfilePath, uuid);
                    if (!ObjectUtils.isEmpty(map)) {
                        cUtil.cmd(map.get("cmd"));
                        //删除生成的ts文件
                        File file1 = new File(map.get("path"));
                        if (file1.exists()) {
                            file1.delete();
                        }
                    }
                } catch (IOException e) {
                    log.error(e.getMessage());
                    e.printStackTrace();
                }
            }
        });
        executor.shutdown();

        return Constants.RESOURCE_PREFIX + "/" + ph;
    }

    /**
     * 实时播放的拼接流
     *
     * @param cameraCode 相机编码
     * @param channel    相机通道
     * @return
     */
    public static String getPlayFlv(String cameraCode, String channel, boolean flay) {
        if (!flay) {
            return bakUrl + "/hdl/" + channel + "/" + cameraCode + ".flv";
        }
        return webUrl + "/hdl/" + channel + "/" + cameraCode + ".flv";


//        if (!flay) {
//            return bakUrl + "/ws/" + channel + "/" + cameraCode + ".flv";
//        }
//        return webUrl + "/ws/" + channel + "/" + cameraCode + ".flv";
    }

    public static String getPlayFlv(String cameraCode, String channel) {
        return getPlayFlv(cameraCode, channel, true);
    }

    /**
     * 开启录制功能
     *
     * @param cameraCode 相机编码
     * @param channel    相机通道
     * @return
     */
    public static String startRecording(String cameraCode, String channel) {
        return historyUrl + "/api/record/flv/start?streamPath=" + channel + "/" + cameraCode;
    }

    /**
     * 关闭录制功能
     *
     * @param taskId 录像接口返回的任务ID
     * @return
     */
    public static String endRecording(String taskId) {
        return historyUrl + "/api/record/flv/stop?id=" + taskId;
    }


    /**
     * 历史回放流（获取）
     *
     * @param channel 相机通道
     * @param startTm 开始时间
     * @param endTm   结束时间
     * @return
     */
    public static String historyPlayListStr(String channel, Date startTm, Date endTm) {
        List<String> list = filterPlayList(channel, startTm, endTm, filePath);
        String uuid = UUID.randomUUID().toString();
        String ph = "flv/" + DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, new Date())
                + "/" + channel + "/"
                + uuid + ".flv";
        if (!ObjectUtils.isEmpty(list)) {
            try {
                return historyPlay(list, ph, uuid);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
        } else {
            throw new BaseException("当前相机无视频录像");
        }
        return null;
    }


    /**
     * 过滤符合条件的视频
     *
     * @param channel
     * @param startTm
     * @param endTm
     * @param mappingUrl
     * @return
     */
    public static List<String> filterPlayList(String channel, Date startTm, Date endTm, String mappingUrl) {
        if (StringUtils.isBlank(channel)
                || ObjectUtils.isEmpty(startTm)
                || ObjectUtils.isEmpty(endTm)) {
            return null;
        }

        List<String> ls = new ArrayList<>();
        Map<Date, String> m = new HashMap<>();
        // 调用视频服务返回参数
        String startTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, startTm);
        String endTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, endTm);

        String param = "channel=" + channel + "&startTime=" + startTime + "&endTime=" + endTime;
        String s = HttpUtils.sendGet(historyUrl + "/api/record/flv/list", param);
        // 视频拼接
        if (!StringUtils.isBlank(s) || "null".equals(s)) {
            List<Map<String, Object>> maps = JSON.parseArray(s, Map.class);
            if (ObjectUtils.isEmpty(maps)) {
                return null;
            }
            for (Map<String, Object> map : maps) {
                Object path = map.get("Path");
                if (!ObjectUtils.isEmpty(path)) {
                    String s1 = path.toString();
                    String substring = s1.substring(s1.lastIndexOf("/") + 1, s1.length());
                    String substring1 = substring.substring(0, substring.indexOf("-"));
                    String[] s2 = substring1.split("_");
                    if (!ObjectUtils.isEmpty(s2)) {

                        String s3 = s2[0] + s2[1];
                        String s4 = s2[0] + s2[2];
                        Date sdate = DateUtils.dateTime(DateUtils.YYYYMMDDHHMMSS, s3);
                        if (s2[1].startsWith("23") && s2[2].startsWith("00")) {
                            sdate = DateUtils.addDays(sdate, -1);
                        }
                        Date edate = DateUtils.dateTime(DateUtils.YYYYMMDDHHMMSS, s4);
                        /*     sdate  |startTm|  edate     |endTm| */
                        if (startTm.compareTo(sdate) >= 0
                                && startTm.compareTo(edate) <= 0
                                && endTm.compareTo(edate) >= 0) {
                            m.put(sdate, mappingUrl + path.toString());
                            /*  |startTm|   sdate    edate   |endTm| */
                        } else if (startTm.compareTo(sdate) <= 0 && endTm.compareTo(edate) >= 0) {
                            m.put(sdate, mappingUrl + path.toString());

                            /*  |startTm|   sdate     |endTm|  edate  */
                        } else if (startTm.compareTo(sdate) <= 0
                                && endTm.compareTo(sdate) >= 0
                                && endTm.compareTo(edate) <= 0) {
                            m.put(sdate, mappingUrl + path.toString());

                            /*  sdate |startTm|      |endTm| edate  */
                        } else if (startTm.compareTo(sdate) >= 0 && endTm.compareTo(edate) <= 0) {
                            m.put(sdate, mappingUrl + path.toString());
                        }
                    }
                }
            }
        }
        if (!ObjectUtils.isEmpty(m) && m.size() > 0) {
            Set<Date> dates = m.keySet();
            // 排序
            dates.stream().parallel().collect(Collectors.toList()).stream().sorted().forEach(d -> {
                ls.add(m.get(d));
            });
            return ls;
        }
        return null;
    }

    /**
     * 合并视频（转化文件）
     *
     * @param fromVideoFileList 视频路径
     * @param newfilePath       生产新的视频文件路径
     * @throws IOException
     */
    public static Map<String, String> myConvetor(List<String> fromVideoFileList,
                                                 String newfilePath,
                                                 String uuid) throws IOException {
        /*
        * for f in *.flv; do echo "file '$f'" >> mylist.txt; done
            ffmpeg -f concat -i mylist.txt -c copy output.flv
        * */

        File file = new File(newfilePath);
        boolean flay = false;
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
            log.info("创建文件夹：{}", file.getParentFile().getPath());
            flay = true;
        }
        log.info("newfilePath：{}", newfilePath);


        StringBuffer sm = new StringBuffer("for f in ");
        String pathStr = null;
        for (int t = 0; t < fromVideoFileList.size(); t++) {
            File ft = new File(fromVideoFileList.get(t));
            if (ft.exists()) {
                if (t != fromVideoFileList.size() - 1) {
//                    sm.append(fromVideoFileList.get(t) + " ");
                    sm.append(fromVideoFileList.get(t).substring(fromVideoFileList.get(t).lastIndexOf("/") + 1, fromVideoFileList.get(t).length()) + " ");
                } else {
//                    sm.append(fromVideoFileList.get(t));
                    sm.append(fromVideoFileList.get(t).substring(fromVideoFileList.get(t).lastIndexOf("/") + 1, fromVideoFileList.get(t).length()));
                    pathStr = fromVideoFileList.get(t).substring(0, fromVideoFileList.get(t).lastIndexOf("/"));
                }
            }
        }
//        String substring = newfilePath.substring(0, newfilePath.lastIndexOf("."));
//        substring = substring + ".txt";

        String substring = uuid + ".txt";
        log.info("临时转化的文件：{}", substring);


        sm.append("; do echo \"file '$f'\" >> " + substring + "; done");
        String commit = sm.toString();
        log.info("合并转化文件的命令：{}", commit);
        Process proc = Runtime.getRuntime().exec("/bin/bash", null, null);
        BufferedReader in = new BufferedReader(new InputStreamReader(proc.getInputStream()));
        PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(proc.getOutputStream())), true);

        try {
            String commit1 = ffmpegPath + " -f concat -safe 0 -i " + pathStr + "/" + substring + " -c copy " + newfilePath;

            log.info("合并视频命令：{}", commit1);
            List<String> commands = new ArrayList<>();
            if (flay) {
                commands.add("chomd -R 777 " + file.getParentFile().getPath());
            }
            //
            commands.add("cd " + pathStr);
            // 合并成一个临时文件
            commands.add(commit);

            // 临时文件转化为 flv
//            commands.add(commit1);
            log.info("录像视频命令：{}", commands);
            for (String line : commands) {
                log.info("命令：{}", line);
                out.println(line);
            }
            out.println("exit");// 这个命令必须执行，否则in流不结束。
            log.info("命令：{}", "exit");
            String rspLine = "";
            while ((rspLine = in.readLine()) != null) {
                log.info("*****:{}", rspLine);
            }
            proc.waitFor();
            Map<String, String> map = new HashMap<>();
            map.put("cmd", commit1);
            map.put("path", pathStr + "/" + substring);
            return map;


        } catch (InterruptedException e) {
            log.error(e.getMessage());
            e.printStackTrace();
        } finally {
            in.close();
            out.close();
            proc.destroy();
        }
        return null;
    }


    @PostConstruct
    public void init() {
        historyUrl = caneraConfig.getHistoryUrl();
        ffmpegPath = caneraConfig.getFfmpegPath();
        filePath = caneraConfig.getFilePath();
        transcribeFilePath = caneraConfig.getTranscribeFilePath();
        webUrl = caneraConfig.getWebUrl();
        bakUrl = caneraConfig.getBakUrl();
        cUtil = cmdCameraUtil;
    }

    /**
     * 定时任务参数flv文件
     *
     * @throws IOException
     * @throws InterruptedException
     */
    public void deleteFlv() throws IOException, InterruptedException {
        SimpleDateFormat s = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.add(calendar.DATE, -1); //把日期往后增加一天,整数  往后推,负数往前移动
        date = calendar.getTime(); //这个时间就是日期往后推一天的结果
        String path = BaseConfig.getProfile() + "/flv/" + DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, date);
        File file = new File(path);
        if (file.exists()) {
            List<String> rspList = new ArrayList<String>();
            Process proc = Runtime.getRuntime().exec("/bin/bash", null, null);
            BufferedReader in = new BufferedReader(new InputStreamReader(proc.getInputStream()));
            PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(proc.getOutputStream())), true);
            String commit = " rm -rf  " + path;
            List<String> commands = new ArrayList<>();

            // 删除
            commands.add(commit);
            log.info("删除昨天的录像视频命令：{}", commands);
            for (String line : commands) {
                out.println(line);
            }
            out.println("exit");// 这个命令必须执行，否则in流不结束。
            String rspLine = "";
            while ((rspLine = in.readLine()) != null) {
                System.out.println(rspLine);
                rspList.add(rspLine);
            }
            int i = proc.waitFor();
            log.info("执行结果：{}", i);

            in.close();
            out.close();
            proc.destroy();
        }
    }


    /**
     * 定时任务：删除超过配置时长的录制视频
     *
     * @throws IOException
     * @throws InterruptedException
     */
    public void deleteFlvExceed() throws IOException, InterruptedException, ParseException {
        //字典中设置的值
        List<SysDictData> sysCameraRecordTime = dictDataMapper.selectDictDataByType("sys_camera_record_time");

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Date date = new Date();
        String format = sdf.format(date);
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(sdf.parse(format));
        //正数，日期天数加
        //负数，日期天数减                         提前45天
        if (!ObjectUtils.isEmpty(sysCameraRecordTime)) {
            String dictValue = sysCameraRecordTime.get(0).getDictValue();
            calendar.add(Calendar.DAY_OF_YEAR, Integer.parseInt("-" + dictValue));
        } else {
            calendar.add(Calendar.DAY_OF_YEAR, -45);
        }
        date = calendar.getTime();
        log.info("过期date======================================" + sdf.format(date));
        //遍历每个进行视频录制的摄像头
        File file = new File(transcribeFilePath);
        if (file.exists() && file.isDirectory()) {
            //获取文件夹中所有的子文件夹和文件
            File[] files = file.listFiles();
            if (!ObjectUtils.isEmpty(files) && files.length > 0) {
                for (File file1 : files) {
                    //   file1=/opt/streams/record/flv/34020000001320000167
                    if (file1.exists() && file1.isDirectory()) {
                        //相机文件
                        File[] fs = file1.listFiles();
                        if (!ObjectUtils.isEmpty(fs) && fs.length > 0) {
                            for (File f : fs) {
                                //    f=/opt/streams/record/flv/34020000001320000167/20230328_192033_192037-91509173-cd5a-11ed-8a42-fa163e4e1e9f.flv
                                //fName=20230328_192033_192037-91509173-cd5a-11ed-8a42-fa163e4e1e9f.flv
                                String fName = f.getName();
                                String[] split = fName.split("_");
                                //没有进行分割长度就是1
                                if (split.length > 1) {
                                    //20230328
                                    String s = split[0];
                                    Date parse = sdf.parse(s);
                                    long fTime = parse.getTime();
                                    long dateTime = date.getTime();
                                    //删除过期文件
                                    if (fTime < dateTime) {
                                        log.info("文件过期时间===========================" + dateTime);
                                        log.info("该文件保存时间===========================" + fTime);
                                        f.delete();
                                        log.info("删除过期视频文件===========================" + fName);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 定时任务：关闭录制视频
     *
     * @throws IOException
     * @throws InterruptedException
     */
    public void closeRecording() throws IOException, InterruptedException, ParseException {
        LambdaQueryWrapper<BaseCameraManagement> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(BaseCameraManagement::getStartRecording, 1);
        wrapper.isNotNull(BaseCameraManagement::getTaskId);
        List<BaseCameraManagement> list = baseCameraManagementService.list(wrapper);
        log.info("list======================================" + list);
        List<String> collect = list.stream().map(BaseCameraManagement::getCameraCode).collect(Collectors.toList());
//        List<String> collect = new ArrayList<>();
//        collect.add("42010001541320001116");
        //遍历每个进行视频录制的摄像头
        File file = new File(transcribeFilePath);
        log.info("closeRecording-file.Name()======================================" + file.getName());
        log.info("closeRecording-file.exists()======================================" + file.exists());
        if (file.exists() && file.isDirectory()) {
            //获取文件夹中所有的子文件夹和文件
            File[] files = file.listFiles();
            if (!ObjectUtils.isEmpty(files) && files.length > 0) {
                for (File file1 : files) {
                    log.info("closeRecording-file1---------------" + file1);
                    log.info("closeRecording-cameraCode---------------" + collect);
                    String substring = file1.toString().substring(file1.toString().lastIndexOf("\\") + 1);
                    if (collect.contains(substring)) {
                        File[] files1 = CameraUtil.getCurFilesList(file1.toString());
                        if (!ObjectUtils.isEmpty(files1) && files1.length > 0) {
                            int length = files1.length;
                            File file2 = files1[length - 1];
                            log.info("closeRecording-file2---------------" + file2);
                            String time = file2.toString().substring(file2.toString().lastIndexOf("\\") + 1);
                            String substring1 = time.substring(0, 15);
                            String substring2 = substring1.substring(0, 4);
                            String substring3 = substring1.substring(4, 6);
                            String substring4 = substring1.substring(6, 8);
                            String substring5 = substring1.substring(9, 11);
                            String substring6 = substring1.substring(11, 13);
                            String substring7 = substring1.substring(13, 15);
                            String date = substring2 + "-" + substring3 + "-" + substring4 + " " + substring5 + ":" + substring6 + ":" + substring7;
                            System.out.println(date);
                            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                            Date parse = sdf.parse(date);
                            //当前时间戳
                            long dades = System.currentTimeMillis();
                            //文件的时间戳
                            long time1 = parse.getTime();
                            //当前时间10分钟之前的时间戳
                            long time2 = dades - 600000L;
                            if (time2 <= time1 && time1 < dades) {
                            } else {
                                LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
                                lw.eq(BaseCameraManagement::getCameraCode, substring);
                                BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
                                log.info("closeRecording-endRecording------------" + baseCameraManagement.getTaskId());
                                String start = HttpUtils.sendGet(CameraUtil.endRecording(baseCameraManagement.getTaskId()));
                                log.info("closeRecording-endRecording------------" + start);
                                if ("success".equals(start)) {
                                    baseCameraManagement.setStartRecording(2);
                                    baseCameraManagement.setTaskId("");
                                } else {
                                    baseCameraManagement.setStartRecording(2);
                                    baseCameraManagement.setTaskId("");
                                    log.error("关闭视频录制异常日志如下:" + start);
                                }
                                baseCameraManagementService.updateById(baseCameraManagement);
                            }
                        } else {
                            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
                            lw.eq(BaseCameraManagement::getCameraCode, substring);
                            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
                            log.info("closeRecording-endRecording------------" + baseCameraManagement.getTaskId());
                            String start = HttpUtils.sendGet(CameraUtil.endRecording(baseCameraManagement.getTaskId()));
                            log.info("closeRecording-endRecording------------" + start);
                            if ("success".equals(start)) {
                                baseCameraManagement.setStartRecording(2);
                                baseCameraManagement.setTaskId("");
                            } else {
                                baseCameraManagement.setStartRecording(2);
                                baseCameraManagement.setTaskId("");
                                log.error("关闭视频录制异常日志如下:" + start);
                            }
                            baseCameraManagementService.updateById(baseCameraManagement);
                        }
                    }
                }
            }
        }
    }

    public static File[] getCurFilesList(String filePath) {
        File path = new File(filePath);
        File[] listFiles = path.listFiles(new java.io.FileFilter() {

            @Override
            public boolean accept(File pathname) {
                if (pathname.isFile()) {
                    return true;
                } else {

                    return false;
                }
            }
        });

        return listFiles;
    }

    public static void execute(String command) {
        try {
            ProcessBuilder process = new ProcessBuilder(command);
            process.inheritIO().start().waitFor();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public static void main(String[] args) throws InterruptedException, ParseException, IOException {
//        CameraUtil cameraUtil = new CameraUtil();
//        cameraUtil.closeRecording();
//        String s = "/opt/streams/record/flv/42010001541320001116/20230403_235506_000506-22e1523b-d170-11ed-8a42-fa163e4e1e9f.flv";
//        String fileStr = s.substring(s.lastIndexOf("/") + 1, s.length());
//        String pathStr = s.substring(0, s.lastIndexOf("/"));
//        System.out.println(fileStr);
//        System.out.println(pathStr);

        String s = "20230403235512";
        Date sdate = DateUtils.dateTime(DateUtils.YYYYMMDDHHMMSS, s);
        sdate = DateUtils.addDays(sdate, -1);
        System.out.println(DateUtils.parseDateToStr(DateUtils.YYYYMMDDHHMMSS, sdate));
    }
}
